import boto3


def get_secret(secret_name: str, region_name: str, profile_name=None):
    # Create a Secrets Manager client
    session = boto3.session.Session(profile_name=profile_name)
    client = session.client(
        service_name='secretsmanager',
        region_name=region_name
    )

    get_secret_value_response = client.get_secret_value(
        SecretId=secret_name
    )

    secret = get_secret_value_response['SecretString']
    return secret



import json

from utils.io_utils import write, get_file_list_from_path
from utils.io_utils import write_lines


def read_json(file_path, encoding="utf-8"):
    with open(file_path, "r", encoding=encoding) as jf:
        return json.load(jf)


def read_jsons_from_path(file_path):
    data = []
    for file in get_file_list_from_path(file_path):
        tmp = read_jsons(file)
        data = data + tmp

    return data


def read_jsons(file_path):
    data = []
    with open(file_path, "r") as jf:
        for line in jf:
            data.append(json.loads(line))

    return data


def read_jsons_from_byte(data_bytes, encoding="utf-8"):
    return [json.loads(line) for line in data_bytes.decode(encoding).split('\n') if line.strip()]


def write_dict_as_json_format(data_dict, file_path):
    json_string = json.dumps(data_dict)
    write(json_string, file_path)


def write_array_of_dict_as_multiple_line_json(data, file_path):
    write_lines([json.dumps(d, ensure_ascii=False) for d in data], file_path)


def transfer_array_of_dict_to_jsons_byte(data):
    return "\n".join([json.dumps(d, ensure_ascii=False) for d in data]).encode("utf-8")


def save_json_to_file(data, output_file):
    """
        designed by Ivy Liu
    """
    with open(output_file, "w", encoding="utf-8") as f:
        json.dump(data, f, ensure_ascii=False)



import logging
import os
from pathlib import Path
from urllib.parse import urlparse

import boto3
from botocore.exceptions import ClientError


def is_key_exists_s3(bucket, key, profile_name=None):
    session = boto3.Session(profile_name=profile_name)
    s3 = session.client("s3")
    try:
        s3.head_object(Bucket=bucket, Key=key)
        print(f"Key: '{key}' found!")
        return True
    except ClientError as e:
        if e.response["Error"]["Code"] == "404":
            print(f"Key: '{key}' does not exist!")
            return False
        else:
            print("Something else went wrong")
            raise


def get_object(bucket, key, profile_name=None):
    session = boto3.Session(profile_name=profile_name)
    s3 = session.client("s3")

    response = s3.get_object(Bucket=bucket, Key=key)
    object_bytes = response['Body'].read()

    return object_bytes


def upload_file_to_s3(file_name, bucket, object_name=None, profile_name=None):
    """Upload a file to an S3 bucket

    :param profile_name:
    :param file_name: File to upload
    :param bucket: Bucket to upload to
    :param object_name: S3 object name. If not specified then file_name is used
    :return: True if file was uploaded, else False
    """

    # If S3 object_name was not specified, use file_name
    if object_name is None:
        object_name = os.path.basename(file_name)

    # Upload the file
    session = boto3.Session(profile_name=profile_name)
    s3_client = session.client("s3")

    try:
        response = s3_client.upload_file(file_name, bucket, object_name)
        print(f"upload [{file_name}] to bucket [{bucket}/{object_name}] succeed")
    except ClientError as e:
        logging.error(e)
        return False

    return True


def write_object(bucket, key, data, profile_name=None):
    """
    Writes in-memory data to an S3 bucket with the specified key.

    :param bucket: The S3 bucket name.
    :param key: The object key in the bucket.
    :param data: The in-memory object data to be uploaded (e.g., bytes, string).
    :param profile_name: AWS named profile for session (optional).
    :return: True if the object was successfully written, otherwise False.
    """
    session = boto3.Session(profile_name=profile_name)
    s3_client = session.client("s3")

    try:
        s3_client.put_object(Bucket=bucket, Key=key, Body=data)
        print(f"Successfully written object to s3://{bucket}/{key}")
        return True
    except ClientError as e:
        print(e)
        return False


def get_file_folders(bucket_name, prefix=""):
    s3_client = boto3.client('s3')

    file_names = []

    paginator = s3_client.get_paginator('list_objects_v2')
    pages = paginator.paginate(Bucket=bucket_name, Prefix=prefix)

    for page in pages:
        for obj in page['Contents']:
            file_names.append(obj["Key"])

    return file_names


def download_files(bucket_name, local_path, file_names, folders):
    for folder in folders:
        folder_path = Path.joinpath(Path(local_path), folder)
        folder_path.mkdir(parents=True, exist_ok=True)

    for file_name in file_names:
        download_file(bucket_name, file_name, local_path)


def download_file(bucket_name, file_name, local_path):
    local_path = Path(local_path)

    s3_client = boto3.client('s3')
    file_path = Path.joinpath(local_path, file_name)
    file_path.parent.mkdir(parents=True, exist_ok=True)

    # file_path = os.path.join(local_path, file_name)
    # print(file_path)
    # os.makedirs(os.path.join(local_path, file_name), exist_ok=True)

    s3_client.download_file(
        bucket_name,
        file_name,
        str(file_path)
    )


def parse_s3_path(s3_path):
    o = urlparse(s3_path, allow_fragments=False)
    return o.netloc, o.path[1:]


def delete_path(bucket, s3_path):
    s3_client = boto3.client('s3')
    file_names, _ = get_file_folders(bucket, s3_path)

    for file_name in file_names:
        s3_client.delete_object(Bucket=bucket, Key=file_name)
        print("success deleted file s3://%s/%s" % (bucket, file_name))



def camel_to_snake(camel_case):
    result = [camel_case[0].lower()]

    for char in camel_case[1:]:
        if char.isupper():
            result.extend(['_', char.lower()])
        else:
            result.append(char)

    return ''.join(result)


def replace_windows_path_to_unix_path(path):
    """
        designed by Ivy Liu
    """
    return path.replace("\\", "/")



import psycopg2


def query(sql, conn):
    cur = execute(sql, conn)
    records = cur.fetchall()
    return records


def update(sql, conn):
    cur = execute(sql, conn)
    return cur.rowcount


def execute(sql, conn):
    cur = conn.cursor()
    cur.execute(sql)

    return cur


def get_connection(dbname, host, port, user, password):
    conn = psycopg2.connect(
        dbname=dbname,
        host=host,
        port=port,
        user=user,
        password=password
    )

    return conn



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

print(requirements)
setup(
    name="tshine73-utils",
    version="0.1.3",
    author="tshine73",
    author_email="fan.steven.chiang@gmail.com",
    description="tshine73 utils",
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.12',
    include_package_data=True,
    install_requires=[
        "psycopg2-binary==2.9.9", 'boto3==1.36.3', 'requests==2.32.3'
    ]
)



from datetime import datetime


def format_datetime(dt, fmt="%Y-%m-%d %H:%M:%S"):
    return dt.strftime(fmt)


def iso_timestamp_to_datetime(ts):
    return datetime.fromisoformat(ts)


def parse_string_to_datetime(date_string, fmt="%Y-%m-%d %H:%M:%S"):
    return datetime.strptime(date_string, fmt)



import os


def write(data, file_name):
    with open(file_name, 'w') as f:
        f.write(data)


def write_lines(lines, file_name):
    with open(file_name, 'w') as f:
        for line in lines:
            f.write(line + "\n")


def delete_file(file_name):
    if os.path.exists(file_name):
        os.remove(file_name)


def read_file(file_name):
    data = []
    with open(file_name, "r") as f:
        for line in f.readlines():
            data.append(line.strip())

    return data


def read_from_bytes(data_bytes, encoding="utf-8"):
    return [line for line in data_bytes.decode(encoding).split('\n') if line.strip()]


def get_file_suffix(file_name):
    _, file_extension = os.path.splitext(file_name)
    return file_extension


def get_file_list_from_path(file_path):
    files = []
    for names in os.listdir(file_path):
        files.append(os.path.join(file_path, names))

    return files


def mkdir_if_not_exists(path):
    if not os.path.isdir(path):
        os.makedirs(path, exist_ok=True)
        print(f"path {path} has been created")


def get_directory_from_path(file_path):
    return os.path.dirname(file_path)


def move_data(src_file, dest_file):
    """Move data from source file to destination file."""
    if not os.path.exists(src_file):
        raise FileNotFoundError(f"source file [{src_file}] does not exist.")

    with open(src_file, "r") as f_src, open(dest_file, "w") as f_dest:
        f_dest.write(f_src.read())

    # Remove source file only after successful write
    os.remove(src_file)
    print(f"source file [{src_file}] has been moved to [{dest_file}].")



import requests


def download(url: str, output_file_name: str, params: dict = None):
    response = requests.get(url, params=params, stream=True)
    if response.status_code == 200:
        with open(output_file_name, "wb") as f:
            f.write(response.content)

        print(f"download {response.url} success")
    else:
        raise Exception(f"Failed to download file from {response.url}: {response.status_code}")


def call_api(url: str, params: dict = None):
    """
    designed by Ivy Liu
    """
    response = requests.get(url, params=params, stream=True)
    if response.status_code == 200:
        return response.json()
    else:
        raise Exception(f"API request failed for {response.url}: {response.status_code}")
